@@ -3,6 +3,7 @@ use std::ops::AddAssign;
 use std::ops::Sub;
 use std::ops::SubAssign;
 
+use crate::num::FromRounded;
 use crate::num::Num;
 use crate::num::ToRounded;
 
@@ -82,26 +83,25 @@ impl<N: Num> Circle<N> {
     pub fn sub_radius(&mut self, n: N) {
         self.1 -= n;
     }
-}
 
-impl<N> Circle<N>
-where
-    N: Num + ToRounded<f32>,
-{
-    pub fn iter_circumference_points(self, num_points: usize) -> CircleCircumferencePointsIterator {
-        CircleCircumferencePointsIterator::new(self, num_points)
+    pub fn abs(&self) -> Self {
+        Self(self.centre().abs(), self.radius().abs())
     }
 
-    pub fn iter_circumference_lines(self, num_lines: usize) -> CircleCircumferenceLinesIterator {
-        CircleCircumferenceLinesIterator::new(self, num_lines)
+    pub fn min(self, other: Self) -> Self {
+        Self(
+            self.centre().min(other.centre()),
+            self.radius().min(other.radius()),
+        )
+    }
+
+    pub fn max(self, other: Self) -> Self {
+        Self(
+            self.centre().max(other.centre()),
+            self.radius().max(other.radius()),
+        )
     }
-}
 
-impl<N> Circle<N>
-where
-    N: Num + ToRounded<f32>,
-    f32: ToRounded<N>,
-{
     pub fn overlaps(&self, other: Self) -> bool {
         let self_rounded = self.to_rounded();
         let other_rounded = other.to_rounded();
@@ -115,6 +115,23 @@ where
     pub fn contains_point(&self, point: Point<N>) -> bool {
         self.centre().hypot_to(point) <= self.radius()
     }
+
+    pub fn iter_circumference_points(self, num_points: usize) -> CircleCircumferencePointsIterator {
+        CircleCircumferencePointsIterator::new(self, num_points)
+    }
+
+    pub fn iter_circumference_lines(self, num_lines: usize) -> CircleCircumferenceLinesIterator {
+        CircleCircumferenceLinesIterator::new(self, num_lines)
+    }
+}
+
+impl Circle<f32> {
+    pub(crate) fn from_f32<N: Num>(self) -> Circle<N> {
+        Circle(
+            self.centre().from_f32(),
+            FromRounded::from_rounded(self.radius()),
+        )
+    }
 }
 
 impl<O: Num, N: Num + ToRounded<O>> ToRounded<Circle<O>> for Circle<N> {
