@@ -18,6 +18,7 @@ use ::num_traits::sign::abs;
 use ::num_traits::sign::signum;
 use ::num_traits::sign::Signed;
 
+use crate::num::FromRounded;
 use crate::num::INum;
 use crate::num::Num;
 use crate::num::NumIdentity;
@@ -78,11 +79,28 @@ impl<N: Num> Point<N> {
         Size::new(other.first() - self.first(), other.second() - self.second())
     }
 
-    /// This is to allow creating a new Point, with a new type, from the type given.
-    /// i.e. `Point::new(1 as u8, 1 as u8)::to::<u32>()`
+    /// This is to allow creating a new Point, which wraps a different number type.
+    ///
+    /// For example `Point(1u8, 2u8)::to::<u32>()` creates a `Point(1u32, 2u32)`.
     pub fn to<T: Num + From<N>>(&self) -> Point<T> {
         Point(T::from(self.first()), T::from(self.second()))
     }
+
+    pub fn abs(&self) -> Self {
+        Self(self.x().abs(), self.y().abs())
+    }
+
+    pub fn min(self, other: Self) -> Self {
+        Self(self.x().min(other.x()), self.y().min(other.y()))
+    }
+
+    pub fn max(self, other: Self) -> Self {
+        Self(self.x().max(other.x()), self.y().max(other.y()))
+    }
+
+    pub fn hypot_to(self, other: Point<N>) -> N {
+        self.distance_to(other).hypot()
+    }
 }
 
 impl<N: Num> NumTuple<N> for Point<N> {
@@ -116,16 +134,6 @@ impl<N: Num> NumTuple<N> for Point<N> {
     }
 }
 
-impl<N> Point<N>
-where
-    N: Num + ToRounded<f32>,
-    f32: ToRounded<N>,
-{
-    pub fn hypot_to(self, other: Point<N>) -> N {
-        self.distance_to(other).hypot()
-    }
-}
-
 impl<O: Num, N: Num + ToRounded<O>> ToRounded<Point<O>> for Point<N> {
     fn to_rounded(self) -> Point<O> {
         Point(self.x().to_rounded(), self.y().to_rounded())
@@ -225,6 +233,13 @@ impl Point<f32> {
     fn angle_to_zero(self) -> f32 {
         self.y().atan2(self.x())
     }
+
+    pub(crate) fn from_f32<N: Num>(self) -> Point<N> {
+        Point(
+            FromRounded::from_rounded(self.x()),
+            FromRounded::from_rounded(self.y()),
+        )
+    }
 }
 
 impl<N: Num> Add<Self> for Point<N> {
